//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* special1: matrix operations; correlation coefficients                  *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://apps.jcns.fz-juelich.de                                         *//
//**************************************************************************//

#include <cstdlib>
#include <cstdio>
#include <cmath>
#include <iostream>
#include <boost/format.hpp>
#include <gsl/gsl_eigen.h>

#include <trivia/mystd.hpp>
#include <read-plus/ask.hpp>

#include "defs.h"
#include "olf.h"
#include "mem.h"
#include "special.h"

using boost::format;

//! Determine eigenvalue and eigenvectors of quadratic data set.

void NSpecial::Eigen() 
{
    throw string( "TODO broken after V rewrite" );
/*
    NOlm::IterateD fiter;
    COld *f0, fout0, fout1;
    CSpec Sout0;
    uint i, n;
    gsl_vector *EVal;
    gsl_matrix *Data, *EVec;
    gsl_eigen_symmv_workspace *Work;

    while((f0=fiter())) {
        if( !( n = f0->nPts() ) ){
            printf( "! Eigen: spectra must have equal length\n" );
            continue;
        } else if ( n != f0->nSpec() ) {
            printf( "! Eigen: file must contain a quadratic matrix\n" );
            continue;
        }

        fout0 = *f0;
        fout1 = *f0;

        fout0.lDoc.push_back( "y := eigenvalues" );
        fout1.lDoc.push_back( "y := eigenvectors" );

        fout0.xco = CCoord("idx", "");
        fout0.yco = CCoord("lambda", "");
        fout0.ZCo.clear();
        fout0.VS.clear();
        fout1.ZCo.clear();
        fout1.ZCo.push_back( CCoord("idx", "") );
            
        EVal = gsl_vector_alloc( n );
        Data = gsl_matrix_alloc( n, n );
        EVec = gsl_matrix_alloc( n, n );
        Work = gsl_eigen_symmv_alloc( n );

        for( uint j=0; j<n; j++ ){
            for( i=0; i<n; i++ ){
                gsl_matrix_set( Data, i, j, f0->VS[j].y[i] );
            }
        }

        gsl_eigen_symmv( Data, EVal, EVec, Work );
        gsl_eigen_symmv_sort( EVal, EVec, GSL_EIGEN_SORT_VAL_DESC  );
        Sout0.clear();
        for( i=0; i<n; ++i ){
            Sout0.push_xy( (double) i, EVal->data[i] );
            printf( "eval %d: %g\n", i, EVal->data[i] );
        }
        fout0.VS.push_back( Sout0 );

        for( uint j=0; j<n; j++ ){
            fout1.VS[j].clear();
            fout1.VS[j].z.push_back( (double) j);
            for( i=0; i<n; i++ ){
                fout1.VS[j].push_xy( (double) i,
                                       gsl_matrix_get( EVec, i, j ) );
            }
        }

        gsl_vector_free( EVal );
        gsl_matrix_free( Data );
        gsl_matrix_free( EVec );
        gsl_eigen_symmv_free( Work );

        NOlm::mem_store( fout0 );
        NOlm::mem_store( fout1 );
    }
*/
};


//! Calculate correlation coefficients.

void NSpecial::CalcCorrel() 
{
    throw string( "TODO broken after V rewrite" );
/*
    NOlm::IterateD fiter;
    COld *f0, *f1, fout;
    CSpec *S0, *S1, S;
    double s0, s1, s00, s11, s01, y0, y1, cc;
    static int k1=-1;
    uint i, j, jj, n, neff, nmin, nmax, ndim;
    static int modeIgnore=0, modeFile=0;
    static double yIgnore=0.;
    bool j2floating;
      if( NRead::stack_empty() )
  cout << "Calculate correlation coefficients between:\n"
        "  (1) x[j] and y[j]\n"
        "  (2) y[j] and y'[j]\n"
        "  (3) y[j] and y[j']\n"
        "  (4) y[j] and y'[j']\n"; 
    modeFile = iask( "Option", modeFile );
    if( modeFile<=0 ) return;

    modeIgnore = iask( "Ignore (0:nothing 1:equal yI 2: below yI",
                       modeIgnore );
    if( modeIgnore !=0 ){
        yIgnore = dask( "yI (ignore when equal/below)", yIgnore );
    }
    
    while((f0=fiter())) {
        
        if( modeFile==2 || modeFile==4 ){
            k1 = k1<0 ? iask( "second file" ) : iask( "second file", k1 );
            NOlm::mem_assert_idx( k1 );
            if( !(f1= P2D(NOlm::MOM[k1])) ){
                printf("! is not a data file\n");
                return;
            }
        } else {
            f1 = f0;
        }
        
        j2floating = modeFile==3 || modeFile==4;
        // dimenionality of output:
        if     ( j2floating && f1->nSpec()>1 )
            ndim = 2;
        else if( f0->nSpec()>1 )
            ndim = 1;
        else
            ndim = 0;
        
        fout = *f0;

        fout->lDoc.push_back( "y := correlation with file "+strg(k1) );
        fout->yco = CCoord("cor["+f0->yco.name+"]", "");
        fout->ZCo.clear();
        if       ( ndim==2 ){
            fout->ZCo.push_back( f0->ZCo[0] );
            fout->xco = f0->ZCo[0];
        } else if( ndim==1 ){
            fout->xco = f0->ZCo[0];
            fout->VS.clear();
            fout->VS.push_back( CSpec() );
        } else {
            // do not save
        }

            
        for( j=0; j<f0->nSpec(); j++ ){

            S0 = &(f0->VS[j]);
            n = S0->size();
            nmin = n;
            nmax = 0;

            if( ndim==2 ){
                fout->VS[j].clear();
                fout->VS[j].z.push_back( S0->z[0] );
            }

            for( jj = (j2floating ? 0 : j);
                 jj < (j2floating ? f1->nSpec() : j+1); ++jj ){

                S1 = &(f1->VS[jj]);

                if( S1->size()!=n ){
                    printf( "S0[j=%i].size=%i not equal S1[jj=%i].size=%u\n",
                            j, n, jj, S1->size() );
                    return;
                }

                s0 = 0; s1 = 0; s00 = 0; s11 = 0; s01 = 0; neff = 0;
                for (i=0; i<n; i++) {
                    y0 = S0->y[i];
                    y1 = modeFile==1 ? S1->x[i] : S1->y[i];
                    if( ( modeIgnore==1 && (y0==yIgnore || y1==yIgnore ) ) ||
                        ( modeIgnore==2 && (y0< yIgnore || y1< yIgnore ) ) )
                        continue;
                    neff += 1;
                    s0 += y0;
                    s1 += y1;
                    s00 += SQR( y0 );
                    s11 += SQR( y1 );
                    s01 += y0 * y1;
                }
                if( neff<nmin ) nmin = neff;
                if( neff>nmax ) nmax = neff;
                cc = neff>0 ? (s01-s0*s1/neff) /
                    sqrt((s00-s0*s0/neff)*(s11-s1*s1/neff)) : 0;

                if        ( ndim==2 ){
                    fout->VS[j].push_xy( S1->z[0], cc);
                } else if ( ndim==1 ) {
                    fout->VS[0].push_xy( S0->z[0], cc);
                } else {
                    printf( "1x1: corr = %g\n", cc );
                }
            }
            printf( "column %d: min %d max %d nonnil correlations\n",
                    j, nmin, nmax );
        }
        if( ndim>0 ) NOlm::mem_store( fout );
}
*/
};


//! Calculate WHAT???

void NSpecial::CalcCorrelDiff() 
{
    throw string( "TODO broken after V rewrite" );
/*
    NOlm::IterateD fiter;
    COld *fin, fout;
    CSpec *S0, *S1, S;
    double s0, s1, s00, s11, s01, y0, y1;
    uint i, n, neff;
    printf(" warning: PISA version: ignore 0.0 data\n" );
    while((fin=fiter())) {
        if( !( n = fin->nPts() ) ){
            printf( "! CalcCorrelDiff: spectra must have equal length\n" );
            printf( "(do not delete invalid items: set them to zero)\n" );
            continue;
        }
        fout = *fin;

        fout->lDoc.push_back( "y := impact on correlation coefficient " );
        fout->yco = CCoord("dr^2["+fin->yco.name+"]", "");
        fout->ZCo.push_back( CCoord( fin->ZCo[0].name+"'", fin->ZCo[0].unit ) );
        fout->ZCo.push_back( CCoord("r^2["+fin->yco.name+"]", "") );
        fout->VS.clear();

        for (uint j0=0; j0<fin->nSpec(); j0++) {
            for (uint j1=0; j1<fin->nSpec(); j1++) {
                S0 = &(fin->VS[j0]);
                S1 = &(fin->VS[j1]);

                S = *S0;

                s0 = 0; s1 = 0; s00 = 0; s11 = 0; s01 = 0; neff = 0;
                for (i=0; i<n; i++) {
                    y0 = S0->y[i];
                    y1 = S1->y[i];
                    if( y0<0 || y1<0 ) continue;
                    neff += 1;
                    s0 += y0;
                    s1 += y1;
                    s00 += SQR( y0 );
                    s11 += SQR( y1 );
                    s01 += y0 * y1;
                }
                
                S.z.push_back( S1->z[0] );
                S.z.push_back( (s01-s0*s1/neff)/sqrt(
                                   (s00-s0*s0/neff)*(s11-s1*s1/neff)) );

                for (i=0; i<n; i++) {
                    y0 = S0->y[i];
                    y1 = S1->y[i];
                    if( y0<0 || y1<0 ){
                        S.y[i] = -1;
                    } else {
                        S.y[i] = ( (s01-y0*y1) - (s0-y0)*(s1-y1)/(neff-1) ) / 
                            sqrt(
                                ( (s00-y0*y0) - (s0-y0)*(s0-y0)/(neff-1) ) *
                                ( (s11-y1*y1) - (s1-y1)*(s1-y1)/(neff-1) ) );
                    }
                }
                
                fout->VS.push_back(S);
            }
        }
        NOlm::mem_store( fout );
    }
*/
};

