extern double func_data_intp_lin( double v, double dk, double dj );

//**************************************************************************//
//*   Functions based on data files                                        *//
//**************************************************************************//

//! linear interpolation in data from spectrum k,j.

double func_data_intp_lin( double v, double dk, double dj )
{
    uint k = lrint( dk );
    uint j = lrint( dj );
    if( k>=NOlm::MOM.size() )
        throw string( "intp: invalid k" );
    POld fd = P2D( NOlm::MOM[k] );
    if( !fd )
        throw "f" + strg(k) + " is no data file";
    if( j>=fd->nJ() )
        throw string( "intp: invalid j" );
    const PSpec s = fd->VS(j);
    // we assume that s->x is sorted
    uint ilow = 0, ihig = s->size()-1;
    if( v < s->x[ilow] || v > s->x[ihig] )
        return 0;
    while( ihig-ilow > 1 ){
        uint imid = (ilow+ihig)/2;
        if( v < s->x[imid] )
            ihig = imid;
        else
            ilow = imid;
    }
    return s->y[ilow] +
        (v-s->x[ilow])/(s->x[ihig]-s->x[ilow])*(s->y[ihig]-s->y[ilow]);
}

