//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  loop.cpp
//! \brief Implements iterator over file selection.

#include "defs.hpp"
#include <algorithm>
#include "../trivia/vector_ops.hpp"
#include "olf.hpp"
#include "ptr.hpp"
#include "mem.hpp"
#include "fsel.hpp"
#include "loop.hpp"

FileIterator::FileIterator( const vector<int>& _Sel, bool tolerate_empty )
{
    Sel = _Sel;
    for( int k: Sel )
        if( k<0 || k>=NOlm::mem_size() )
            throw "invalid entry " + S(k) + " in file selection";
    if ( !tolerate_empty && !size() )
        throw S("empty file selection");
    reset();
}

POlo FileIterator::get()
{
    if ( iV>=Sel.size() )
        return POlo();
    int k=Sel[iV];
    if( k<0 )
        throw S("BUG: k<0 in file selection");
    if( k>=NOlm::mem_size() )
        throw "FileSelection["+S(iV)+"]="+S(k)+" exceeds online memory size "+S(NOlm::mem_size());
    ++iV;
    return NOlm::mem_get(k);
}

POld FileIterator::getD()
{
    POlo f = get();
    if( !f )
        return POld();
    if( POld ret = P2D(f) )
        return ret;
    throw S("BUG: non-D file found by D iterator");
}

POlc FileIterator::getC()
{
    POlo f = get();
    if( !f )
        return POlc();
    if( POlc ret = P2C(f) )
        return ret;
    throw S("BUG: non-C file found by C iterator");
}
